#include "sinx.h"

//
// Writer implementation
//

SINX_WriteFileOpen::SINX_WriteFileOpen (const std::string &aFilename)
{
	m_Filename = aFilename;
	m_File = fopen (aFilename.c_str (),"wb");
	m_OpenOk = (m_File!=NULL);
	// write some initial stuff
	//WriteSpecialComment ("This file is compliant with SINX level 1\n SINX format home page: http://mikle33.narod.ru/sinx");
	WriteCrLf ();
}

SINX_WriteFileOpen::~SINX_WriteFileOpen ()
{
	if (m_OpenOk)
	{
		fclose (m_File);
		// if writing ended in an error, delete the file
		if (CheckError ()) unlink (m_Filename.c_str ());
	}
}

bool SINX_WriteFileOpen::WriteData
	(const SINX1_Platform::TChar *pData,unsigned aCharCount)
{
	bool aResult = true;
	for (; aCharCount>0; aCharCount--, pData++)
	{
		if ((unsigned)*pData<32 && *pData!='\r' && *pData!='\n')
			aResult &= WriteCharAsCode (*pData);
		else
			aResult &= (fputc (*pData,m_File)!=EOF);
	}
	return aResult;
}

bool SINX_WriteFileOpen::WriteCrLf ()
{
	return WriteData ("\r\n",2); // SINX1_Platform::TChar is char
}

//
// Reader implementation
//

SINX_ReadFileOpen::SINX_ReadFileOpen (const std::string &aFilename):
	SINX1_Symbol (),
	m_OpenOk (false)
{
	m_File = fopen (aFilename.c_str (),"rb");
	if (m_File==NULL)
	{
		m_OpenOk = false;
		return;
	}
	else
		m_OpenOk = true;

	fseek (m_File,0,SEEK_END);
	SINX1_Platform::Buffer *pData =
		SINX1_Platform::Buffer::AllocL (ftell (m_File));
	fseek (m_File,0,SEEK_SET);
	fread ((char *)pData->Data (),1,pData->Length (),m_File);
	fclose (m_File);
	m_File = NULL;

	(SINX1_Symbol &)(*this) = SINX1_Symbol (pData);
}

SINX_ReadFileOpen::~SINX_ReadFileOpen ()
{
	if (m_File!=NULL) fclose (m_File); // a fallback precaution
}

//
// helpers
//

SINX::StdString SINX::ReadSpecialStringL
	(const SINX::Symbol &aFrom,const SINX::StdString &aDefault)
{
	return aFrom.IsEof ()? aDefault : aFrom.DataAsStringL (aDefault);
}

void SINX::WriteSpecialStringL (SINX::WriteNode &aTo,const TChar *pNewSpecName,
	const StdString &aData,int aHints)
{
	aTo.WriteSpecial (pNewSpecName,aData,aHints);
}

int SINX::ReadSpecialIntL (const SINX::Symbol &aFrom,int aDefault)
{
	return aFrom.IsEof ()? aDefault : aFrom.DataAsIntL (aDefault);
}

void SINX::WriteSpecialIntL (SINX::WriteNode &aTo,const TChar *pNewSpecName,
	int aData,int aHints)
{
	aTo.WriteSpecialInt (pNewSpecName,aData,aHints);
}

int SINX::ReadSpecialUnsignedL (const SINX::Symbol &aFrom,unsigned aDefault)
{
	return aFrom.IsEof ()? aDefault : aFrom.DataAsIntL (aDefault);
}

void SINX::WriteSpecialUnsignedL (SINX::WriteNode &aTo,
	const TChar *pNewSpecName,unsigned aData,int aHints)
{
	aTo.WriteSpecialUnsigned (pNewSpecName,aData,aHints);
}

bool SINX::ReadSpecialBoolL (const SINX::Symbol &aFrom,bool aDefault)
{
	if (aFrom.IsEof ()) return aDefault;
	SINX::StdString aStr = aFrom.DataAsStringL ("");
	if (aStr=="off") return false;
	else if (aStr=="on") return true;
	else return aDefault;
}

void SINX::WriteSpecialBoolL (SINX::WriteNode &aTo,const TChar *pNewSpecName,
	bool aData,int aHints)
{
	if (aData) aTo.WriteSpecial (pNewSpecName,"on",aHints);
	else aTo.WriteSpecial (pNewSpecName,"off",aHints);
}
